;;;; -*-  Mode: LISP; Syntax: Common-Lisp; Base: 10                          -*-
;;;; ---------------------------------------------------------------------------
;;;; File name: indirect.lsp
;;;;    System: LTRE
;;;;   Version: 4.1
;;;;    Author: Ken Forbus, Johan de Kleer
;;;;   Created: May 1, 1986
;;;;  Modified: everett on Sat Jul 6 20:24:43 1996
;;;;   Purpose: Indirect proof mechanism for LTRE
;;;; Copyright: 1986, 1989, 1990, 1991, 1995 Kenneth D. Forbus, 
;;;;            Northwestern University, and Johan de Kleer, Xerox Corporation.
;;;;            All Rights Reserved.
;;;; ---------------------------------------------------------------------------

(in-package :LTRE)

(proclaim '(special *LTRE*))

(defun try-indirect-proof (fact &optional (*LTRE* *LTRE*))
  (unless (known? fact)
    (with-contradiction-handler (ltre-ltms *ltre*)
      #'(lambda (contradictions ltms &aux assumptions)
	  (setq assumptions
		(assumptions-of-clause
		 (car contradictions)))
	  (let ((the-node
		 (find (datum-tms-node (referent fact T))
		       assumptions)))
	    (when the-node
	      (let ((status (tms-node-label the-node)))
		(retract-assumption the-node)
		(add-nogood the-node status
			    assumptions)))))
      ;; Assume the negation
       (assuming `((:NOT ,fact)) *LTRE*
		 (run-rules)))
   (known? fact)))

;;; Example of indirect proof

(defun indirect-proof-example ()
  (in-ltre (create-ltre "Indirect Proof Example"))
  (assert! '(:OR p q) 'user)
  (assert! '(:IMPLIES p r) 'user)
  (assert! '(:IMPLIES q r) 'user)
  (known? 'r)
  (try-indirect-proof 'r)
  (known? 'r))
