;;;; -*-  Mode: LISP; Syntax: Common-Lisp; Base: 10                          -*-
;;;; ---------------------------------------------------------------------------
;;;; File name: ltrecgex.lsp
;;;;    System: LTRE
;;;;   Version: 4.1
;;;;    Author: John Everett
;;;;   Created: June 1, 1996
;;;;  Modified: everett on Fri Aug 2 9:55:26 1996
;;;;   Purpose: LTRE Common Graphics Extensions
;;;; Copyright: 1986, 1989, 1990, 1991, 1995 Kenneth D. Forbus, 
;;;;            Northwestern University, and Johan de Kleer, Xerox Corporation.
;;;;            All Rights Reserved.
;;;; ---------------------------------------------------------------------------

(in-package :LTRE)


;;; A COMMON-GRAPHICS-BASED INTERFACE TO ACTIVATING DEBUGGING INSTRUMENTATION
;;; ----------------------------------------------------------------------------
;;; NOTE: These functions are specific to Franz Allegro Common Lisp for the PC, 
;;; and will not work under other Lisps.  Although they may run under version 
;;; 2.0 of ACLPC, they were developed under version 3.0.
;;; Given that you have ACLPC, load this file and call (debug-ltre) to pop up a 
;;; dialog showing all possible debugging flags.

#+COMMON-GRAPHICS
(defun debug-ltre ()
   (make-ltre-debugging-dialog))

#+COMMON-GRAPHICS
(defun add-flags (widget new old)
   (declare (ignore new old))
   (let* ((dialog (dialog-item-dialog widget))
          (flag-list (find-named-object :all-debugging-flags dialog))
          (selections (dialog-item-value flag-list))
          (actives (find-named-object :active-flags dialog)))
      (dolist (flag selections)
         (unless (member flag (dialog-item-range actives))
            (list-widget-add-item actives flag nil))
         (pushnew flag (ltre-dbg-flags *ltre*)))
      (setf (ltre-debugging *ltre*) T)))

#+COMMON-GRAPHICS
(defun remove-flags (widget new old)
   (declare (ignore new old))
   (let* ((dialog (dialog-item-dialog widget))
          (actives (find-named-object :active-flags dialog))
          (active-range (dialog-item-range actives))
          (selections (dialog-item-value actives))
          (new-actives (nset-difference active-range selections)))
      (set-dialog-item-range actives new-actives)
      (setf (ltre-dbg-flags *ltre*) new-actives)))

#+COMMON-GRAPHICS
(defun close-debugging-dialog (widget new old)
   (close (dialog-item-dialog widget)))

#+COMMON-GRAPHICS
(defun make-ltre-debugging-dialog (&aux dbg-dialog)
   (setq dbg-dialog
      (open-dialog 
         (list 
            (make-dialog-item 
               :widget 'static-text 
               :name :static-text-5 
               :title (delete #\Newline "All LTRE Debugging Flags") 
               :value nil 
               :box (make-box 24 32 209 50) 
               :tabstop nil 
               :groupstart t 
               :background-color t 
               :font (make-font nil :arial 16 '(:bold)))
            (make-dialog-item 
               :widget 'multi-item-list 
               :name :all-debugging-flags
               :title nil 
               :value nil
               :box (make-box 24 54 209 252) 
               :tabstop t 
               :background-color white
               :groupstart t 
               :key 'capitalize-object 
               :range (sort (copy-list *ltre-dbg-flags*)
                         #'string<
                         :key #'string-downcase) 
               :font (make-font nil :arial 16 nil))
            (make-dialog-item 
               :widget 'multi-item-list 
               :name :active-flags
               :title (delete #\Newline "Active Flags") 
               :value nil
               :box (make-box 284 54 469 252) 
               :tabstop t 
               :groupstart t 
               :key 'capitalize-object 
               :background-color white
               :range (ltre-dbg-flags *ltre*)
               :font (make-font nil :arial 16 nil) 
               :tab-control nil)
            (make-dialog-item 
               :widget 'static-text 
               :name :static-text-6 
               :title (delete #\Newline "Active Flags") 
               :value nil
               :box (make-box 284 32 469 50) 
               :tabstop nil 
               :groupstart t 
               :background-color t 
               :font (make-font nil :arial 16 '(:bold)) 
               :tab-control nil)
            (make-dialog-item 
               :widget 'default-button 
               :name :OK-button
               :title "OK"
               :set-value-fn 'close-debugging-dialog
               :box (make-box 135 286 209 326) 
               :tabstop nil 
               :groupstart nil 
               :font (make-font nil :arial 16 '(:bold)))
            (make-dialog-item 
               :widget 'cancel-button
               :name :cancel-button
               :title "Cancel"
               :set-value-fn 'close-debugging-dialog
               :box (make-box 284 286 358 326) 
               :tabstop nil 
               :groupstart nil 
               :font (make-font nil :arial 16 '(:bold)) 
               :tab-control nil)
            (make-dialog-item 
               :widget 'button 
               :name :add-flags
               :title ">>>"
               :box (make-box 223 102 264 142) 
               :tabstop nil 
               :groupstart nil 
               :set-value-fn 'add-flags
               :font (make-font nil :arial 16 '(:bold)))
            (make-dialog-item 
               :widget 'button 
               :name :remove-flags
               :title "<<<"
               :box (make-box 223 157 264 197) 
               :tabstop nil 
               :groupstart nil 
               :set-value-fn 'remove-flags
               :font (make-font nil :arial 16 '(:bold)) 
               :tab-control nil))
         'dialog 
         *lisp-main-window* 
         :name :ltre-debugging-controller 
         :title "LTRE Debugging Controller" 
         :font (make-font :swiss :system 16 '(:bold)) 
         :window-state :normal
         :window-border :frame 
         :left-attachment nil 
         :top-attachment nil 
         :right-attachment nil 
         :bottom-attachment nil 
         :user-movable t 
         :user-resizable t 
         :user-closable t 
         :user-shrinkable t 
         :user-scrollable nil 
         :overlapped nil 
         :background-color (make-rgb :red 192 :green 192 :blue 192) 
         :pop-up-p nil 
         :window-interior (make-box 189 216 688 555)))
   (setf (window-editable-p dbg-dialog) t)
   (setf (getf (stream-plist dbg-dialog) :path) 
         "c:\\qrg\\ltre\\v4a\\ltre-dbg.lsp")
   (setf (getf (stream-plist dbg-dialog) :startup-state) nil)
   (setf (getf (stream-plist dbg-dialog) :top-level-p) nil)
   (setf (help-string dbg-dialog) (delete #\Newline nil))
   (setf (getf (stream-plist dbg-dialog) :package) nil)
   (let* ((box (getf *window-exteriors* (object-name dbg-dialog))))
      (when box (reshape-window-exterior dbg-dialog box)))
   (show-window dbg-dialog nil)
   dbg-dialog)
;;; ----------------------------------------------------------------------------
;;; END OF CODE